import org.jetbrains.kotlin.gradle.tasks.KotlinCompile

plugins {
	id("org.springframework.boot") version "3.1.3"
	id("io.spring.dependency-management") version "1.1.2"
	kotlin("jvm") version "1.9.0"
	kotlin("plugin.spring") version "1.8.22"
	id("io.gitlab.arturbosch.detekt") version("1.23.1")
	id("org.sonarqube") version "4.4.1.3373"
	id("jacoco")
}

detekt {
	toolVersion = "1.23.1"
	config.setFrom(file("$rootDir/src/main/resources/detekt/config.yml"))
	buildUponDefaultConfig = true
}

group = "br.pucrs.ages.victims"
version = "0.0.1-SNAPSHOT"

val keycloakVersion = "22.0.1"
val junitVersion = "5.9.2"
val mockkVersion = "1.13.4"

java {
	sourceCompatibility = JavaVersion.VERSION_17
}

repositories {
	mavenCentral()
}

dependencies {
	implementation("org.jetbrains.kotlin:kotlin-reflect")
	implementation("com.fasterxml.jackson.module:jackson-module-kotlin")
	// spring
	implementation("org.springframework.boot:spring-boot-starter-webflux")
	implementation("org.springframework.boot:spring-boot-starter-actuator")
	implementation("org.springframework.boot:spring-boot-starter-security")
	implementation("org.springframework.boot:spring-boot-starter-web")
	implementation("org.springframework.boot:spring-boot-starter-validation")
	implementation("org.springdoc:springdoc-openapi-starter-webmvc-ui:2.1.0")
	implementation("org.springframework.cloud:spring-cloud-starter-openfeign:4.0.2")
	// security
	implementation("org.keycloak:keycloak-admin-client:$keycloakVersion")
	implementation("org.springframework.boot:spring-boot-starter-security")
	implementation("org.springframework.boot:spring-boot-starter-oauth2-resource-server")
	// tests
	testImplementation("org.junit.jupiter:junit-jupiter:$junitVersion")
	testImplementation("io.mockk:mockk:$mockkVersion")
	testImplementation("org.springframework.boot:spring-boot-starter-test")
}

tasks.withType<KotlinCompile> {
	kotlinOptions {
		freeCompilerArgs += "-Xjsr305=strict"
		jvmTarget = "17"
	}
}

tasks.withType<Test> {
	useJUnitPlatform()
	systemProperty("spring.profiles.active", "test")
}

jacoco {
	toolVersion = "0.8.8"
}

val exclusions = arrayOf(
	"com/generate/**",
	"br/pucrs/ages/victims/api/web/controller/**",
	"br/pucrs/ages/victims/api/web/error/**",
	"br/pucrs/ages/victims/api/web/representation/**",
	"br/pucrs/ages/victims/api/mapper/**",
	"br/pucrs/ages/victims/api/config/**",
	"br/pucrs/ages/victims/api/exception/**",
	"br/pucrs/ages/victims/api/repository/**",
)

tasks.withType<JacocoReport> {
	dependsOn("test")
	sourceSets(sourceSets.main.get())
	executionData(fileTree(project.rootDir.absolutePath).include("**/build/jacoco/*.exec"))
	reports {
		xml.required = true
		xml.outputLocation =
			layout.buildDirectory.file("$buildDir/reports/jacoco/jacocoTestReport/jacocoTestReport.xml")
		csv.required = false
		html.outputLocation = layout.buildDirectory.dir("jacocoHtml")
	}
	finalizedBy(tasks.jacocoTestCoverageVerification)

	classDirectories.setFrom(classDirectories.files.map {
		fileTree(it).matching {
			exclude(*exclusions)
		}
	})

}

tasks.withType<JacocoCoverageVerification> {
	violationRules {
		rule {
			element = "BUNDLE"
			classDirectories.setFrom(tasks.jacocoTestCoverageVerification.get().classDirectories)
			limit {
				counter = "INSTRUCTION"
				value = "COVEREDRATIO"
				minimum = BigDecimal(0.5)

			}
		}
	}
}

sonar {
	properties {
		property("sonar.projectKey", "vitimas-de-crime_auth")
		property("sonar.organization", "vitimas-de-crime")
		property(
			"sonar.coverage.jacoco.xmlReportPaths",
			"$buildDir/reports/jacoco/jacocoTestReport/jacocoTestReport.xml"
		)
		property("sonar.coverage.exclusions", exclusions.joinToString(", "))
	}
}