package br.pucrs.ages.victims.auth.repository.keycloak

import br.pucrs.ages.victims.auth.config.properties.KeycloakProperties
import br.pucrs.ages.victims.auth.domain.User
import br.pucrs.ages.victims.auth.domain.dto.Page
import br.pucrs.ages.victims.auth.exception.NotFoundException
import br.pucrs.ages.victims.auth.mapper.UserMapper
import br.pucrs.ages.victims.auth.repository.UserRepository
import org.keycloak.admin.client.Keycloak
import org.keycloak.admin.client.resource.UsersResource
import org.keycloak.representations.idm.RoleRepresentation
import org.keycloak.representations.idm.UserRepresentation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Repository
import java.util.*
import kotlin.math.ceil

@Repository
class UserRepositoryImpl(
    @Autowired
    val keycloak: Keycloak,
    @Autowired
    private val userMapper: UserMapper,
    @Autowired
    val keycloakProperties: KeycloakProperties
) : UserRepository {

    private fun findUserRepresentationById(id: UUID): UserRepresentation =
        usersResource()
            .searchByAttributes("$APPLICATION_ID:$id").stream()
            .findFirst()
            .orElseThrow { NotFoundException("Usuário não encontrado.") }

    override fun findById(id: UUID): User {
        return findUserRepresentationById(id)
            .let { userMapper.fromRepresentationToDomain(it) }
    }

    override fun findByCpf(cpf: String): Optional<User> {
        return usersResource().search(cpf).stream()
            .findFirst()
            .map { userMapper.fromRepresentationToDomain(it) }
    }

    override fun findByEmail(email: String): Optional<User> {
        return usersResource().searchByEmail(email, true).stream()
            .findFirst()
            .map { userMapper.fromRepresentationToDomain(it) }
    }

    override fun save(user: User): User {
        usersResource().create(userMapper.toRepresentation(user))

        return user
    }

    override fun updatePasswordById(id: UUID, password: String) {
        val user = findUserRepresentationById(id)

        usersResource().get(user.id)
            .resetPassword(userMapper.mapPasswordToKeycloakCredential(
                password = password,
                isTemporary = false
            ))
    }

    override fun findAll(nameFilter: String?, active: Boolean?, pageNumber: Int, pageSize: Int): Page<User> {
        val users = searchUsers(nameFilter, active, pageNumber, pageSize)
            .filter { it.email != null }

        val totalElements = countUsers(nameFilter, active).toLong()

        return Page(
            content = users.stream()
                .map { userMapper.fromRepresentationToDomain(it) }
                .toList(),
            pageNumber = pageNumber,
            pageSize = pageSize,
            totalElements = totalElements,
            totalPages = ceil(totalElements.toDouble() / pageSize).toInt()
        )
    }

    private fun searchUsers(
        nameFilter: String?,
        active: Boolean?,
        pageNumber: Int,
        pageSize: Int
    ): MutableList<UserRepresentation> =
        usersResource().search(
            null,
            nameFilter,
            null,
            null,
            pageNumber * pageSize,
            pageSize,
            active,
            false
        )


    private fun countUsers(nameFilter: String?, active: Boolean?): Int =
        usersResource().count(
            null,
            null,
            nameFilter,
            null,
            true,
            null,
            active,
            null
        )


    private fun usersResource(): UsersResource =
        keycloak.realm(keycloakProperties.realm)
            .users()

    override fun deleteById(id: UUID) {
        val user = findUserRepresentationById(id)

        keycloak.realm(keycloakProperties.realm).users().get(user.id).update(
            UserRepresentation().apply {
                isEnabled = false
            }
        )
    }
    override fun addRoleById(id: UUID, roleName: String) {
        val user = findUserRepresentationById(id)

        val userResource = usersResource().get(user.id.toString())
        userResource
            .roles()
            .realmLevel()
            .add(listOf(findRoleByName(roleName)))
    }

    private fun findRoleByName(roleName: String): RoleRepresentation =
        keycloak.realm(keycloakProperties.realm)
            .roles()
            .get(roleName)
            .toRepresentation()
}