package br.pucrs.ages.victims.auth.service

import br.pucrs.ages.victims.auth.domain.User
import br.pucrs.ages.victims.auth.domain.dto.Page
import br.pucrs.ages.victims.auth.domain.dto.UserDto
import br.pucrs.ages.victims.auth.exception.ExistingResourceException
import br.pucrs.ages.victims.auth.exception.ValidationException
import br.pucrs.ages.victims.auth.repository.UserRepository
import br.pucrs.ages.victims.auth.service.external.InviteService
import br.pucrs.ages.victims.auth.validator.UserValidator
import br.pucrs.ages.victims.auth.web.representation.request.RegisterAdminRequest
import br.pucrs.ages.victims.auth.web.representation.request.RegisterUserRequest
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import java.time.LocalDateTime
import java.util.*

private const val ADMIN_ROLE = "ADMIN"

@Service
class UserService(
    @Autowired
    private val userRepository: UserRepository,
    @Autowired
    private val userValidator: UserValidator,
    @Autowired
    private val inviteService: InviteService,
    @Autowired
    private val authService: AuthService
) {

    fun getCurrentUser(): UserDto {
        val user = authService.getAuthenticatedUser()
        val userInfo = userRepository.findById(user.id)

        return UserDto(
            id = user.id,
            name = userInfo.name,
            email = userInfo.email,
            cpf = userInfo.cpf,
            phoneNumber = userInfo.phoneNumber,
            active = userInfo.active,
            roles = user.roles
        )
    }

    fun findById(id: UUID): User {
        return userRepository.findById(id)
    }

    fun create(payload: RegisterUserRequest): User {
        val userFoundByCpf = userRepository.findByCpf(payload.cpf)

        if (userFoundByCpf.isPresent) {
            throw ExistingResourceException("Um usuário com esse CPF já está cadastrado")
        }

        val userFoundByEmail = userRepository.findByEmail(payload.email)

        if (userFoundByEmail.isPresent) {
            throw ExistingResourceException("Um usuário com esse email já está cadastrado")
        }

        userValidator.validatePayload(payload)

        val newUser = User(
            email = payload.email,
            name = payload.name,
            cpf = payload.cpf,
            phoneNumber = payload.phoneNumber,
            active = false
        )

        userRepository.save(newUser)
        userRepository.updatePasswordById(newUser.id, payload.password)

        return newUser
    }

    fun findAll(nameFilter: String?, active: Boolean?, pageNumber: Int? = 0, pageSize: Int? = 10): Page<User> {
        return userRepository.findAll(nameFilter, active, pageNumber!!, pageSize!!)
    }

    fun delete(id: UUID) {
        val user = userRepository.findById(id)
        if (!user.active) {
            throw ExistingResourceException("Usuário já deletado")
        }

        userRepository.deleteById(id)
    }

    fun createAdmin(payload: RegisterAdminRequest): User {
        userValidator.validateAdminPayload(payload)

        val invite = inviteService.findById(UUID.fromString(payload.inviteId))

        if (!invite.active || LocalDateTime.now().isAfter(invite.expirationDate)) {
            throw ValidationException("O convite informado não é válido")
        }

        val user = create(RegisterUserRequest(
            name = payload.name,
            email = invite.receiverEmail,
            cpf = payload.cpf,
            phoneNumber = "",
            password = payload.password,

        ))
        userRepository.addRoleById(user.id, ADMIN_ROLE)

        val accessToken = "Bearer ${authService.login(user.email, payload.password).accessToken}"
        inviteService.markInviteAsUsed(UUID.fromString(payload.inviteId), user.id, accessToken)

        return user
    }

}