package br.pucrs.ages.victims.auth.service

import br.pucrs.ages.victims.auth.domain.dto.Page
import br.pucrs.ages.victims.auth.exception.ExistingResourceException
import br.pucrs.ages.victims.auth.exception.ValidationException
import br.pucrs.ages.victims.auth.fixtures.aJwtTokenDTO
import br.pucrs.ages.victims.auth.fixtures.aUser
import br.pucrs.ages.victims.auth.fixtures.aUserPrincipal
import br.pucrs.ages.victims.auth.fixtures.anInviteDto
import br.pucrs.ages.victims.auth.repository.UserRepository
import br.pucrs.ages.victims.auth.service.external.InviteService
import br.pucrs.ages.victims.auth.validator.UserValidator
import br.pucrs.ages.victims.auth.web.representation.request.RegisterAdminRequest
import br.pucrs.ages.victims.auth.web.representation.request.RegisterUserRequest
import io.mockk.*
import org.junit.jupiter.api.Assertions.assertEquals
import org.junit.jupiter.api.Assertions.assertNotNull
import org.junit.jupiter.api.Nested
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import java.util.*

class UserServiceTest {

    private val userRepository = mockk<UserRepository>()
    private val userValidator = mockk<UserValidator>()
    private val inviteService = mockk<InviteService>()
    private val authService = mockk<AuthService>()

    private val userService = UserService(
        userRepository = userRepository,
        userValidator = userValidator,
        inviteService = inviteService,
        authService = authService
    )

    @Nested
    inner class GetCurrentUser {

        @Test
        fun `should return current user`() {
            val user = aUser()
            val userPrincipal = aUserPrincipal(user)

            every { authService.getAuthenticatedUser() } returns userPrincipal
            every { userRepository.findById(user.id) } returns user

            val currentUser = userService.getCurrentUser()

            assertEquals(user.id, currentUser.id)
            assertEquals(user.name, currentUser.name)
            assertEquals(user.email, currentUser.email)
            assertEquals(user.cpf, currentUser.cpf)
            assertEquals(user.phoneNumber, currentUser.phoneNumber)
            assertEquals(user.active, currentUser.active)
            assertEquals(userPrincipal.roles, currentUser.roles)
        }
    }

    @Nested
    inner class FindById {

        @Test
        fun `should return user when valid id is provided`() {
            val id = UUID.randomUUID()
            val user = aUser()

            every { userRepository.findById(any()) } returns user

            val userFound = userService.findById(id)

            assertEquals(user, userFound)
            verify { userRepository.findById(id) }
        }
    }

    @Nested
    inner class Create {

        @Test
        fun `should create user when valid payload is provided`() {
            val payload = RegisterUserRequest("teste", "12345678911", "OAKODOab@1", "OAKODOab@1")
            val mockedUser = aUser()

            every { userValidator.validatePayload(any()) } just runs
            every { userRepository.findByCpf(any()) } returns Optional.empty()
            every { userRepository.updatePasswordById(any(), any()) } just runs
            every { userRepository.save(any()) } returns mockedUser
            every { userRepository.findByEmail(any()) } returns Optional.empty()

            val user = userService.create(payload)

            assertNotNull(user.id)
            verify { userRepository.save(any()) }
        }

        @Test
        fun `should throw ExistingResourceException when user with CPF already exists`() {
            val payload = RegisterUserRequest("teste", "12345678911", "OAKODOab@1", "OAKODOab@1")
            every { userRepository.findByCpf(any()) } returns Optional.of(aUser())

            assertThrows<ExistingResourceException>("Um usuário com esse CPF já está cadastrado") {
                userService.create(payload)
            }
        }

        @Test
        fun `should throw ExistingResourceException when user with email already exists`() {
            val payload = RegisterUserRequest("teste", "12345678911", "OAKODOab@1gmail.com", "OAKODOab@1")
            every { userRepository.findByCpf(any()) } returns Optional.empty()
            every { userRepository.findByEmail(any()) } returns Optional.of(aUser())

            assertThrows<ExistingResourceException>("Um usuário com esse email já está cadastrado") {
                userService.create(payload)
            }
        }
    }


    @Nested
    inner class Delete {

        @Test
        fun `should delete user when valid id is provided`() {
            val user = aUser().copy(active = true)
            every { userRepository.findById(any()) } returns user
            every { userRepository.deleteById(any()) } just runs

            userService.delete(user.id)
            every { userRepository.findByCpf(any()) } returns Optional.empty()
            every { userRepository.findByEmail(any()) } returns Optional.empty()
            every { userValidator.validatePayload(any()) } throws ValidationException("Senha inválida")

            verify { userRepository.deleteById(user.id) }
        }

        @Test
        fun `should throw ExistingResourceException when user is already deleted`() {
            val id = UUID.randomUUID()
            val user = aUser().copy(active = false)
            every { userRepository.findById(any()) } returns user

            assertThrows<ExistingResourceException>("Usuário já deletado") { userService.delete(id) }
        }
    }

    @Nested
    inner class FindAll {

        @Test
        fun `should return a page of users`() {
            val nameFilter = "teste"
            val pageNumber = 0
            val pageSize = 10
            val active = true
            val mockedPage =
                Page(content = listOf(aUser()), pageNumber = 0, pageSize = 10, totalElements = 1, totalPages = 1)

            every { userRepository.findAll(nameFilter, active, pageNumber, pageSize) } returns mockedPage

            val page = userService.findAll(nameFilter, active, pageNumber, pageSize)

            assertNotNull(page)
            verify { userRepository.findAll(nameFilter, active, pageNumber, pageSize) }
        }
    }

    @Nested
    inner class CreateAdmin {

        @Test
        fun `should create admin user`() {
            val payload = RegisterAdminRequest(
                name = "teste",
                cpf = "12345678911",
                password = "OAKODOab@1",
                inviteId = "123e4567-e89b-12d3-a456-426614174000"
            )

            every { userValidator.validateAdminPayload(payload) } just runs
            every { inviteService.findById(any()) } returns anInviteDto()
            every { userValidator.validatePayload(any()) } just runs
            every { userRepository.findByCpf(any()) } returns Optional.empty()
            every { userRepository.findByEmail(any()) } returns Optional.empty()
            every { userRepository.save(any()) } answers { firstArg() }
            every { userRepository.updatePasswordById(any(), any()) } just runs
            every { userRepository.addRoleById(any(), any()) } just runs
            every { authService.login(any(), any()) } returns aJwtTokenDTO()
            every { inviteService.markInviteAsUsed(any(), any(), any()) } just runs


            userService.createAdmin(payload)

            verify { userRepository.addRoleById(any(), "ADMIN") }
            verify { inviteService.markInviteAsUsed(any(), any(), any()) }
        }

        @Test
        fun `should throw ValidationException when invite is invalid`() {
            val payload = RegisterAdminRequest(
                name = "teste",
                cpf = "12345678911",
                password = "OAKODOab@1",
                inviteId = "123e4567-e89b-12d3-a456-426614174000"
            )

            every { userValidator.validateAdminPayload(payload) } just runs
            every { inviteService.findById(any()) } returns anInviteDto(active = false)

            assertThrows<ValidationException>("O convite informado não é válido") { userService.createAdmin(payload) }

            verify(exactly = 0) { userRepository.addRoleById(any(), "ADMIN") }
        }
    }

}